@@ -10,8 +10,8 @@ from fdsreader.isof import Isosurface
 from fdsreader.isof.IsosurfaceCollection import IsosurfaceCollection
 from fdsreader.part import Particle
 from fdsreader.part.ParticleCollection import ParticleCollection
-from fdsreader.plot3d import Plot3D
-from fdsreader.plot3d.Plot3dCollection import Plot3DCollection
+from fdsreader.pl3d import Plot3D
+from fdsreader.pl3d.Pl3dCollection import Plot3DCollection
 from fdsreader.slcf import Slice
 from fdsreader.slcf.SliceCollection import SliceCollection
 from fdsreader.utils import Mesh, Dimension, Surface, Quantity, Ventilation, Extent
@@ -112,7 +112,7 @@ class Simulation:
                         offsets = smv_file.readline().strip().split()
                         self.default_texture_origin = tuple(float(offsets[i]) for i in range(3))
                     elif keyword == "CLASS_OF_PARTICLES":
-                        self.particles.append(self.register_particle(smv_file))
+                        self.particles.append(self._register_particle(smv_file))
                     elif "GRID" in keyword:
                         self.meshes.append(self._load_mesh(smv_file, keyword))
                     elif keyword == "SURFACE":
@@ -130,7 +130,7 @@ class Simulation:
                         self._load_slice(smv_file, keyword)
                     elif "ISOG" in keyword:
                         self._load_isosurface(smv_file, keyword)
-                    elif keyword == "PL3D":
+                    elif "PL3D" in keyword:
                         self._load_data_3d(smv_file, keyword)
                     elif "BND" in keyword:
                         self._load_boundary_data(smv_file, keyword)
@@ -442,7 +442,8 @@ class Simulation:
                     if obst_id not in patches:
                         patches[obst_id] = list()
                     patches[obst_id].append(p)
-                patch_offset += fdtype.new((('f', str(p.dimension.shape(cell_centered=False))),)).itemsize
+                patch_offset += fdtype.new(
+                    (('f', str(p.dimension.shape(cell_centered=False))),)).itemsize
 
         for obst_id, p in patches.items():
             for patch in p:
@@ -451,7 +452,7 @@ class Simulation:
                                                     p, times, n_t, lower_bounds, upper_bounds)
 
     def _load_data_3d(self, smv_file: TextIO, line: str):
-        """Loads the plot3d at current pointer position.
+        """Loads the pl3d at current pointer position.
         """
         # Todo: Also read SMOKG3D data?
         line = line.strip().split()
@@ -507,7 +508,7 @@ class Simulation:
                                                       double_quantity, quantity, label, unit)
             self.isosurfaces[iso_id]._add_subsurface(self.meshes[mesh_index], iso_filename)
 
-    def register_particle(self, smv_file: TextIO) -> Particle:
+    def _register_particle(self, smv_file: TextIO) -> Particle:
         particle_class = smv_file.readline().strip()
         color = tuple(float(c) for c in smv_file.readline().strip().split())
 
@@ -520,26 +521,29 @@ class Simulation:
             quantities.append(Quantity(quantity, label, unit))
         return Particle(particle_class, quantities, color)
 
-    def _load_particle_meta(self, particles: List[Particle], file_path: str) -> List[float]:
+    def _load_particle_meta(self, particles: Union[List[Particle], ParticleCollection],
+                            file_path: str, mesh: Mesh) -> List[float]:
         with open(file_path, 'r') as bnd_file:
             line = bnd_file.readline().strip().split()
             n_classes = int(line[1])
             times = list()
             n_q = list()
-            for _ in range(n_classes):
+            for i in range(n_classes):
                 line = bnd_file.readline().strip().split()
                 n_q.append(int(line[0]))
                 for _ in range(n_q[-1]):
                     bnd_file.readline()
+                particles[i].n_particles[mesh] = list()
             bnd_file.seek(0)
 
             for line in bnd_file:
                 times.append(float(line.strip().split()[0]))
                 for i in range(n_classes):
-                    bnd_file.readline()
+                    particle = particles[i]
+                    particle.n_particles[mesh].append(
+                        int(bnd_file.readline().strip().split()[1].strip()))
                     for q in range(n_q[i]):
                         line = bnd_file.readline().strip().split()
-                        particle = particles[i]
                         quantity = particle.quantities[q].quantity
                         particle.lower_bounds[quantity].append(float(line[0]))
                         particle.upper_bounds[quantity].append(float(line[1]))
@@ -548,12 +552,13 @@ class Simulation:
     def _load_particle_data(self, smv_file: TextIO, line: str):
         file_path = os.path.join(self.root_path, smv_file.readline().strip())
 
-        if type(self.particles) == list:
-            times = self._load_particle_meta(self.particles, file_path + '.bnd')
-            self.particles = ParticleCollection(times, self.particles)
-
         mesh_index = int(line.split()[1].strip()) - 1
         mesh = self.meshes[mesh_index]
+
+        times = self._load_particle_meta(self.particles, file_path + '.bnd', mesh)
+        if type(self.particles) == list:
+            self.particles = ParticleCollection(times, self.particles)
+
         self.particles._file_paths[mesh] = file_path
 
         n_classes = int(smv_file.readline().strip())
